(* Copyright (C) 2000, HELM Team.
 * 
 * This file is part of HELM, an Hypertextual, Electronic
 * Library of Mathematics, developed at the Computer Science
 * Department, University of Bologna, Italy.
 * 
 * HELM is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * HELM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HELM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA  02111-1307, USA.
 * 
 * For details, see the HELM World-Wide-Web page,
 * http://cs.unibo.it/helm/.
 *)

(******************************************************************************)
(*                                                                            *)
(*                               PROJECT HELM                                 *)
(*                                                                            *)
(*                Claudio Sacerdoti Coen <sacerdot@cs.unibo.it>               *)
(*                                 24/01/2000                                 *)
(*                                                                            *)
(* This module is the terms level of a parser for cic objects from xml        *)
(* files to the internal representation. It is used by the module cicParser2  *)
(* (objects level). It defines an extension of the standard dom using the     *)
(* object-oriented extension machinery of markup: an object with a method     *)
(* to_cic_term that returns the internal representation of the subtree is     *)
(* added to each node of the dom tree                                         *)
(*                                                                            *)
(******************************************************************************)

exception IllFormedXml of int

val ids_to_targets : (Cic.id, Cic.anntarget) Hashtbl.t option ref
val current_sp : string list ref
val current_uri : UriManager.uri ref
val process_annotations : bool ref

(* the "interface" of the class linked to each node of the dom tree *)
class virtual cic_term :
  object ('a)

    (* fields and methods ever required by markup *)
    val mutable node : cic_term Pxp_document.node option
    method clone : 'a
    method node : cic_term Pxp_document.node
    method set_node : cic_term Pxp_document.node -> unit

    (* a method that returns the internal representation of the tree (term) *)
    (* rooted in this node                                                  *)
    method virtual to_cic_term : Cic.annterm

  end

(* The definition of domspec, an hashtable that maps each node type to the *)
(* object that must be linked to it. Used by markup.                       *)
val domspec : cic_term Pxp_document.spec
