(* Copyright (C) 2000, HELM Team.
 * 
 * This file is part of HELM, an Hypertextual, Electronic
 * Library of Mathematics, developed at the Computer Science
 * Department, University of Bologna, Italy.
 * 
 * HELM is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * HELM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HELM; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA  02111-1307, USA.
 * 
 * For details, see the HELM World-Wide-Web page,
 * http://cs.unibo.it/helm/.
 *)

(******************************************************************************)
(*                                                                            *)
(*                               PROJECT HELM                                 *)
(*                                                                            *)
(*                Claudio Sacerdoti Coen <sacerdot@cs.unibo.it>               *)
(*                                 14/06/2000                                 *)
(*                                                                            *)
(*                                                                            *)
(******************************************************************************)

let get_annotation_from_term annterm =
 let module C = Cic in
  match annterm with
     C.ARel (_,ann,_,_)             -> ann
   | C.AVar (_,ann,_)               -> ann
   | C.AMeta (_,ann,_)              -> ann
   | C.ASort (_,ann,_)              -> ann
   | C.AImplicit (_,ann)            -> ann
   | C.ACast (_,ann,_,_)            -> ann
   | C.AProd (_,ann,_,_,_)          -> ann
   | C.ALambda (_,ann,_,_,_)        -> ann
   | C.ALetIn (_,ann,_,_,_)         -> ann
   | C.AAppl (_,ann,_)              -> ann
   | C.AConst (_,ann,_,_)           -> ann
   | C.AAbst (_,ann,_)              -> ann
   | C.AMutInd (_,ann,_,_,_)        -> ann
   | C.AMutConstruct (_,ann,_,_,_,_)-> ann
   | C.AMutCase (_,ann,_,_,_,_,_,_) -> ann
   | C.AFix (_,ann,_,_)             -> ann
   | C.ACoFix (_,ann,_,_)           -> ann
;;

let get_annotation_from_obj annobj =
 let module C = Cic in
  match annobj with
     C.ADefinition (_,ann,_,_,_,_)        -> ann
   | C.AAxiom (_,ann,_,_,_)               -> ann
   | C.AVariable (_,ann,_,_,_)            -> ann
   | C.ACurrentProof (_,ann,_,_,_,_)      -> ann
   | C.AInductiveDefinition (_,ann,_,_,_) -> ann
;;

exception IdUnknown of string;;

let get_annotation (annobj,ids_to_targets) xpath =
 try
  match Hashtbl.find ids_to_targets xpath with
     Cic.Object annobj -> get_annotation_from_obj annobj
   | Cic.Term annterm -> get_annotation_from_term annterm
 with
  Not_found -> raise (IdUnknown xpath)
;;
